<?xml version='1.0'?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"  xmlns:camed="http://jcam.org.uk/editor" xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:look="http://jcam.org.uk/LookupLists">
	<xsl:output method="xhtml" indent="yes" doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN" doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"/>
<xsl:template match="/">
<html xmlns="http://www.w3.org/1999/xhtml">
 <head>
 <title>CAM Documentation</title>
 <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"/>
 <link rel="stylesheet" type="text/css" href="fulldoc.css"/>
 </head>
 <body>
	<xsl:apply-templates/>
 </body>
</html>
</xsl:template>


<xsl:template match="as:CAM">
	<div class="CAM">
		<p  class="CAMLevel">CAM Level Supported:<xsl:value-of select="@CAMlevel"/></p>
		<p class="CAMVersion">CAM Version:<xsl:value-of select="@version"/></p>
	</div>
	<xsl:apply-templates select="as:Header"/>
	<xsl:apply-templates select="as:AssemblyStructure"/>
	<xsl:apply-templates select="as:Extension/look:LookupList"/>
</xsl:template>

<xsl:template match="as:Header">
	<div class="CAMHeader">
	<p >Description:<xsl:value-of select="as:Description"/></p>
	<p >Owner:<xsl:value-of select="as:Owner"/></p>
	<p >Version:<xsl:value-of select="as:Version"/></p>
	<p >Date:<xsl:value-of select="as:DateTime"/></p>
	<div class="headerParams">
	<xsl:apply-templates select="as:Parameters"/>
	</div>
	</div>
</xsl:template>

<xsl:template match="as:Parameters">
	<xsl:apply-templates select="as:Parameter"/>
</xsl:template>

<xsl:template match="as:Parameter">
	<div class="Parameter">
		<xsl:apply-templates select="@*" mode="para"/>
	</div>
</xsl:template>



<xsl:template match="@*" mode="para">
<xsl:if test="string-length(.)>0">
<p class="{local-name(..)}-{local-name()}"><xsl:value-of select="local-name()"/>:<xsl:value-of select="."/></p>	
</xsl:if>
</xsl:template>



<xsl:template match="as:Structure">
	<div class="structure">
	<xsl:apply-templates select="@*" mode="para"/>
	<xsl:apply-templates select="as:Element">
		<xsl:with-param name="level" select="1"/>
	</xsl:apply-templates>
	</div>
</xsl:template>

<xsl:template match="as:Element">
<xsl:param name="level"/>
	<div>
	   <xsl:choose>
	   <xsl:when test="count(as:Element) > 0">
	   	<xsl:attribute name="class">Element Parent</xsl:attribute>
	   </xsl:when>
	   <xsl:otherwise>
	   <xsl:attribute name="class">Element Leaf</xsl:attribute>
	   </xsl:otherwise>
	   </xsl:choose>
		<p class="depth"><xsl:value-of select="$level"/></p>
		<xsl:apply-templates select="@*" mode="para"/>
				<xsl:apply-templates select="camed:annotation"/>
		<xsl:apply-templates select="as:Rule/as:constraint"/>

		<xsl:apply-templates select="as:Attribute">
			<xsl:with-param name="level" select="$level"/>
		</xsl:apply-templates>
			<xsl:apply-templates select="as:Element">
		<xsl:with-param name="level" select="$level+1"/>
	</xsl:apply-templates>
	</div>
</xsl:template>

<xsl:template match="as:Attribute">
<xsl:param name="level"/>
	<div class="Attribute">
		<p class="depth"><xsl:value-of select="$level"/></p>
		<p class="Attribute-name">@<xsl:value-of select="@name"/></p>
		<xsl:apply-templates select="@*[not(local-name()='name')]" mode="para"/>
		<xsl:apply-templates/>
	</div>
</xsl:template>


<xsl:template match="as:constraint">
	<div>
	  <xsl:if test="not(@condition)">
	    <xsl:attribute name="class">Constraint</xsl:attribute>
	  </xsl:if>
	  <xsl:if test="./@condition">
	    <xsl:attribute name="class">conditional</xsl:attribute>
	  </xsl:if>
		<xsl:apply-templates select="@*" mode="para"/>
		<xsl:apply-templates select="as:annotation"/>
	</div>
</xsl:template>

<xsl:template match="as:annotation">
	<xsl:for-each select="as:documentation">
	<div class="ruleDocumentation">
		<xsl:apply-templates select="@*" mode="para"/>
		<p><xsl:value-of select="."/></p>
	</div>
	</xsl:for-each>
</xsl:template>

<xsl:template match="camed:annotation">
	<xsl:for-each select="camed:documentation">
	<div class="structureDocumentation">
		<xsl:apply-templates select="@*" mode="para"/>
		<p><xsl:value-of select="." disable-output-escaping="yes"/></p>
	</div>
	</xsl:for-each>
</xsl:template>


<xsl:template match="look:LookupList">
 <div class="lookupList">
	<xsl:apply-templates select="@*" mode="para"/>
	<xsl:for-each select="look:item">
		<div class="lookupItem">
			<xsl:apply-templates select="@*" mode="para"/>
		</div>
	</xsl:for-each>
 </div>
</xsl:template>
</xsl:stylesheet><!-- Stylus Studio meta-information - (c) 2004-2007. Progress Software Corporation. All rights reserved.
<metaInformation>
<scenarios ><scenario default="yes" name="Scenario1" userelativepaths="yes" externalpreview="no" url="fulldoc.xml" htmlbaseurl="" outputurl="fulldoc.html" processortype="saxon8" useresolver="yes" profilemode="0" profiledepth="" profilelength="" urlprofilexml="" commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" validateoutput="no" validator="internal" customvalidator=""/></scenarios><MapperMetaTag><MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/><MapperBlockPosition></MapperBlockPosition><TemplateContext></TemplateContext><MapperFilter side="source"></MapperFilter></MapperMetaTag>
</metaInformation>
-->