<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
        xmlns:xs="http://www.w3.org/2001/XMLSchema" 
        xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
        xmlns:as="http://www.oasis-open.org/committees/cam" 
        xmlns:camed="http://jcam.org.uk/editor"
                        xmlns:w2e="http://thisSheet" 
                        xmlns:cam2func="http://jcam.org.uk/editor/function"
                        exclude-result-prefixes="xs xsd camed w2e cam2func" version="2.0">

        <!-- Provided "as is" License: Creative Commons - http://creativecommons.org/licenses/by-sa/3.0/
     Author: David RR Webber, February, 2008 -->
        <!--
    Purpose  : NDR validation functions used by validate-cam-cxf.xsl - allows external customization of common NDR terms and rule checking.
    Original : 04/18/2009
    Initials : drrw
    CAM ver    : 1.1 
    Description : Part of a set of tools to ease alignment of XSD based validations and documentation and CAM templates.
    Updated  - Inits - Change
    04/18/09   drrw    Initial release
    04/22/09   drrw    Add namespace domain check function
    04/23/09   drrw    Add name type name correlation functions
    04/27/09   drrw    Add email as a representation term
    04/29/09   drrw    Fix html <li> formatting issue in called function
    05/04/09   drrw    Correct inverse logic on domain to ignore check
    08/17/10   drrw    Add numeric type name correlation function
    08/17/10   drrw    Enhance name type correlations
    08/23/10   drrw    Fix uppercase name on numeric type check
    08/24/10   drrw    Replace numeric rule check function
    01/02/11   drrw    Add Key and KeyRef as reference terms, also XPath, Abbreviation, Encoding, Role, Order
    01/02/11   drrw    Also ratio and uid as terms    
    04/26/11   drrw    Refine type / name consistency checks with compound tests - e.g. number and telephone of type string; value and text (LEXS 4)
    04/21/12   drrw    Add Image, Average, Week, and Total to reference terms
    04/21/12   drrw    Change Days, Hours, Minutes to contains in reference terms
    09/14/12   drrw    Enhance for SQL column renaming properties
    10/03/12   drrw    Add Context to representation terms
    03/20/12   drrw    Add XML and Score to representation terms
      -->
   
   <xsl:function name="cam2func:has-representation-term">
      <xsl:param name="name" />
      <xsl:choose>
         <xsl:when test="ends-with($name,'Age')"/>
         <xsl:when test="contains($name,'Amount')"/>
         <xsl:when test="contains($name,'Average')"/>
         <xsl:when test="contains($name, 'Abbreviation')"/>
         <xsl:when test="contains($name, 'Binary')"/>
         <!-- xsl:when test="contains($name, 'BinaryObject')"/ -->
         <xsl:when test="ends-with($name,'Count')"/>
         <xsl:when test="starts-with($name,'Count')"/>
         <xsl:when test="ends-with($name,'Counted')"/>
         <xsl:when test="ends-with($name,'Total')"/>
         <xsl:when test="starts-with($name,'Total')"/>
         <xsl:when test="ends-with($name,'Gender')"/>
         <xsl:when test="contains($name, 'Graphic')"/>
         <xsl:when test="contains($name, 'Picture')"/>
         <xsl:when test="contains($name, 'Image')"/>
         <xsl:when test="contains($name, 'Sound')"/>
         <xsl:when test="contains($name, 'Video')"/>
         <xsl:when test="contains($name, 'Code')"/>
         <xsl:when test="contains($name, 'Category')"/>
         <xsl:when test="contains($name, 'Context')"/>
         <xsl:when test="contains($name, 'Currency')"/>
         <xsl:when test="contains(upper-case($name), 'EMAIL')"/>
         <xsl:when test="contains($name, 'DateTime')"/>
         <xsl:when test="contains($name, 'Date')"/>
         <xsl:when test="contains($name, 'Time')"/>
         <xsl:when test="contains($name, 'Days')"/>
         <xsl:when test="contains($name, 'Week')"/>
         <xsl:when test="contains($name, 'Hours')"/>
         <xsl:when test="contains($name, 'Minutes')"/>
         <xsl:when test="contains($name, 'Indicator')"/>
         <xsl:when test="contains($name, 'Encoding')"/>
         <xsl:when test="contains($name, 'Format')"/>
         <xsl:when test="contains($name, 'Length')"/>
         <xsl:when test="contains($name, 'Width')"/>
         <xsl:when test="contains($name, 'Height')"/>
         <xsl:when test="contains($name, 'Weight')"/>
         <xsl:when test="contains($name, 'Level')"/>
         <xsl:when test="contains($name, 'Location')"/>
         <xsl:when test="contains($name, 'Measure')"/>
         <xsl:when test="contains($name, 'Mode')"/>
         <xsl:when test="contains($name, 'Method')"/>
         <xsl:when test="contains($name, 'Numeric')"/>
         <xsl:when test="contains($name, 'Number')"/>
         <xsl:when test="ends-with(lower-case($name), 'number')"/>
         <xsl:when test="ends-with(lower-case($name), 'name')"/>
         <xsl:when test="contains($name, 'Telephone')"/>
         <xsl:when test="contains($name, 'Address')"/>
         <xsl:when test="contains($name, 'Street')"/>
         <xsl:when test="contains($name, 'City')"/>
         <xsl:when test="contains($name, 'ZIP')"/>
         <xsl:when test="ends-with($name, 'Order')"/>
         <xsl:when test="contains($name, 'Price')"/>
         <xsl:when test="ends-with($name, 'Ratio')"/>
         <xsl:when test="contains($name, 'Role')"/>
         <xsl:when test="contains($name, 'State')"/>
         <xsl:when test="contains($name, 'Status')"/>
         <xsl:when test="contains($name, 'Rank')"/>
         <xsl:when test="contains($name, 'Flag')"/>
         <xsl:when test="contains($name, 'Frequency')"/>
         <xsl:when test="contains($name, 'Format')"/>
         <xsl:when test="contains($name, 'Size')"/>
         <xsl:when test="contains($name, 'Unit')"/>
         <xsl:when test="contains($name, 'Value')"/>
         <xsl:when test="ends-with($name, 'Votes')"/>
         <xsl:when test="ends-with($name, 'Score')"/>
         <xsl:when test="contains($name, 'Version')"/>
         <xsl:when test="contains($name, 'Rate')"/>
         <xsl:when test="contains($name, 'Required')"/>
         <xsl:when test="contains($name, 'Percent')"/>
         <xsl:when test="contains($name, 'Quantity')"/>
         <xsl:when test="contains($name, 'Qty')"/>
         <xsl:when test="contains($name, 'Description')"/>
         <xsl:when test="contains($name, 'Comment')"/>
         <xsl:when test="contains($name, 'Reason')"/>
         <xsl:when test="contains($name, 'Instructions')"/>
         <xsl:when test="contains($name, 'Text')"/>
         <xsl:when test="contains($name, 'Title')"/>
         <xsl:when test="contains($name, 'Type')"/>
         <xsl:when test="contains($name, 'Year')"/>
         <xsl:when test="contains($name, 'Month')"/>
         <xsl:when test="contains($name, 'Day')"/>
         <xsl:when test="contains($name, 'Name')"/>
         <xsl:when test="ends-with($name, 'ID')"/>
         <xsl:when test="ends-with($name, 'UID')"/>
         <xsl:when test="ends-with($name, 'Id')"/>
         <xsl:when test="$name = 'id'"/>
         <xsl:when test="$name = 'uid'"/>
         <xsl:when test="ends-with($name, ':id')"/>
         <xsl:when test="ends-with($name, ':uid')"/>
         <xsl:when test="ends-with($name, 'XML')"/>
         <xsl:when test="ends-with($name, 'Identifier')"/>
         <xsl:when test="ends-with($name, 'Identification')"/>
         <xsl:when test="ends-with($name, 'Key')"/>
         <xsl:when test="ends-with($name, 'KeyRef')"/>
         <xsl:when test="contains($name, 'XPath')"/>
         <xsl:when test="contains($name, 'xpath')"/>
         <xsl:when test="contains($name, 'URI')"/>
         <xsl:when test="contains($name, 'URL')"/>
         <xsl:when test="contains($name, 'URN')"/>
         <xsl:otherwise><xsl:value-of select="concat($name,'\|')"/></xsl:otherwise>
      </xsl:choose>
   </xsl:function>     
   
   <xsl:function name="cam2func:rules2type">
      <xsl:param name="actionRule"/>
      <xsl:choose>
         <xsl:when test="contains($actionRule,'makeNillable')"/><!-- ignore -->
         <xsl:when test="contains($actionRule,',token')">token|</xsl:when>
         <xsl:when test="contains($actionRule,',Token')">token|</xsl:when>
         <xsl:when test="contains($actionRule,'setDateMask')">date|</xsl:when>
         <xsl:when test="contains($actionRule,'setNumberMask')">number|</xsl:when>
         <xsl:when test="contains($actionRule,'base64Binary')">binary|</xsl:when>
         <xsl:when test="$actionRule='setLength(0-1)'">flag|</xsl:when>
         <xsl:when test="contains($actionRule,'restrictValues')">code|</xsl:when>
         <xsl:when test="contains($actionRule,'setLength(')">
            <xsl:choose>
               <xsl:when test="contains($actionRule,'(1)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'(2)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'(3)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'(4)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'(5)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'(6)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'-1)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'-2)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'-3)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'-4)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'-5)')">code|</xsl:when>
               <xsl:when test="contains($actionRule,'-6)')">code|</xsl:when>
               <xsl:otherwise>text|</xsl:otherwise>
            </xsl:choose>
          </xsl:when>
         <xsl:otherwise>string|</xsl:otherwise>
      </xsl:choose>
   </xsl:function>
   
   <xsl:function name="cam2func:type2term">
      <xsl:param name="typeValue"/>
      <xsl:choose>
         <xsl:when test="$typeValue='text'"><xsl:value-of select="'Text'"/></xsl:when>
         <xsl:when test="$typeValue='date'"><xsl:value-of select="'Date'"/></xsl:when>
         <xsl:when test="$typeValue='time'"><xsl:value-of select="'Time'"/></xsl:when>
         <xsl:when test="$typeValue='number'"><xsl:value-of select="'Value'"/></xsl:when>
         <xsl:when test="$typeValue='binary'"><xsl:value-of select="'BinaryObject'"/></xsl:when>
         <xsl:when test="$typeValue='code'"><xsl:value-of select="'Code'"/></xsl:when>
         <xsl:when test="$typeValue='flag'"><xsl:value-of select="'Flag'"/></xsl:when>
         <xsl:when test="$typeValue='token'"><xsl:value-of select="'Code'"/></xsl:when>
         <xsl:when test="$typeValue='string'"><xsl:value-of select="'Text'"/></xsl:when>
         <xsl:otherwise/>
      </xsl:choose>
   </xsl:function>
   
   <xsl:function name="cam2func:computeNDRscore">
      <xsl:param name="NDRresults"/>
      <xsl:variable name="rule1" select="if(contains($NDRresults,'anyAttribute:-')) then 1 else 0"></xsl:variable>
      <xsl:variable name="rule2" select="if(contains($NDRresults,'anySimpleType:-')) then 1 else 0"></xsl:variable>
      <xsl:variable name="rule3" select="if(contains($NDRresults,'anyType:-')) then 1 else 0"></xsl:variable>
      <xsl:variable name="rule4" select="if(contains($NDRresults,'any:-')) then 1 else 0"></xsl:variable>
      <xsl:variable name="rule5" select="if(contains($NDRresults,'choice:-')) then 0.5 else 0"></xsl:variable>
      <xsl:variable name="rule6" select="if(contains($NDRresults,'noAnnoAttributes:-0')) then 0 else 0.5"></xsl:variable>
      <xsl:variable name="rule7" select="if(contains($NDRresults,'elements:-0')) then 0 else 0.5"></xsl:variable>
      <xsl:variable name="rule8" select="if(contains($NDRresults,'attributenames:-0')) then 0 else 0.5"></xsl:variable>
      <xsl:variable name="rule9" select="if(contains($NDRresults,'elementnames:-0')) then 0 else 0.5"></xsl:variable>
      <xsl:variable name="rule10" select="if(contains($NDRresults,'elementnamechar:-0')) then 0 else 0.5"></xsl:variable>
      <xsl:variable name="rule11" select="if(contains($NDRresults,'attributenamechar:-0')) then 0 else 0.5"></xsl:variable>
      <xsl:variable name="rule12" select="if(contains($NDRresults,'attributenamerepr:-0')) then 0 else 0.5"></xsl:variable>
      <xsl:variable name="rule13" select="if(contains($NDRresults,'elementnamerepr:-0')) then 0 else 0.5"></xsl:variable>
      <xsl:value-of select="xsd:decimal(10.0 - $rule1 - $rule2 - $rule3 - $rule4 - $rule5 - $rule6 - $rule7 - $rule8 - $rule9 - $rule10 - $rule11 - $rule12 - $rule13)"/><xsl:value-of> out of 10.0</xsl:value-of>
   </xsl:function>
   
   <xsl:function name="cam2func:nameTypeCorrelation">
      <xsl:param name="name" />
      <xsl:param name="context"/>
      <xsl:choose>
         <xsl:when test="ends-with($name,'CODE')"/>
         <xsl:when test="ends-with($name,'CD')"/>
         <xsl:when test="ends-with($name,'ID')"/>
         <xsl:when test="ends-with($name,'TYPE')"/>
         <xsl:when test="contains($name,'IND')"/>
         <xsl:when test="contains($name,'IDENTIFIER')"/>
         <xsl:when test="contains($name,'NUMBER') and(not(contains($name,'PHONE')))"/>
         <xsl:when test="contains($name,'NUM')"/>
         <xsl:when test="contains($name,'QUANTITY')"/>
         <xsl:when test="contains($name,'QTY')"/>
         <xsl:when test="contains($name,'SIZE')"/>
         <xsl:when test="contains($name,'AMOUNT')"/>
         <xsl:when test="contains($name,'TOTAL')"/>
         <xsl:when test="contains($name,'HEIGHT')"/>
         <xsl:when test="contains($name,'PRICE')"/>
         <xsl:when test="contains($name,'PERCENT')"/>
         <xsl:when test="contains($name,'FREQUENCY')"/>
         <xsl:when test="ends-with($name,'RATE')"/>
         <xsl:when test="ends-with($name,'COUNT')"/>
         <xsl:when test="contains($name,'LENGTH')"/>
         <xsl:when test="contains($name,'VALUE') and(not(contains($name,'TEXT')))"/>
         <xsl:when test="contains($name,'DATE')"/>
         <xsl:when test="contains($name,'TIME')"/>
         <xsl:when test="contains($name,'MEASURE') and(not(contains($name,'COUNTERMEASURE')))"/>
         <xsl:when test="ends-with($name,'FLAG')"/>
         <xsl:otherwise><xsl:value-of select="$context"/></xsl:otherwise>
      </xsl:choose>
   </xsl:function>     
   
   <xsl:function name="cam2func:numericItemsCorrelation">
      <xsl:param name="name" />
      <xsl:param name="context"/>
      <xsl:variable name="thisName" select="upper-case($name)"/>
      <xsl:choose>
         <xsl:when test="contains($thisName,'NUMBER') and(not(contains($thisName,'PHONE')))"/>
         <xsl:when test="contains($thisName,'NUMERIC')"/>
         <xsl:when test="ends-with($thisName,'NUM') and(not(contains($thisName,'PHONE')))"/>
         <xsl:when test="contains($thisName,'QUANTITY')"/>
         <xsl:when test="contains($thisName,'QTY')"/>
         <xsl:when test="ends-with($thisName,'SIZE')"/>
         <xsl:when test="contains($thisName,'AMOUNT')"/>
         <xsl:when test="contains($thisName,'AVERAGE')"/>
         <xsl:when test="contains($thisName,'TOTAL')"/>
         <xsl:when test="contains($thisName,'COUNT')"/>
         <xsl:when test="contains($thisName,'VALUE') and(not(contains($name,'TEXT')))"/>
         <xsl:when test="contains($thisName,'WEIGHT')"/>
         <xsl:when test="contains($thisName,'HEIGHT')"/>
         <xsl:when test="contains($thisName,'PRICE')"/>
         <xsl:when test="contains($thisName,'PERCENT')"/>
         <xsl:when test="contains($thisName,'FREQUENCY')"/>
         <xsl:when test="ends-with($thisName,'RATE')"/>
         <xsl:when test="ends-with($thisName,'COUNT')"/>
         <xsl:when test="contains($thisName,'LENGTH')"/>
         <xsl:when test="contains($thisName,'MEASURE') and(not(contains($thisName,'COUNTERMEASURE')))"/>
         <xsl:otherwise><xsl:value-of select="$context"/></xsl:otherwise>
      </xsl:choose>
   </xsl:function>     
   
   <xsl:function name="cam2func:numericRuleMatch">
      <xsl:param name="xpath" />
      <xsl:param name="context"/>
      <xsl:choose>
         <xsl:when test="contains($xpath,'/*')"/><!-- ignore choice xpath rules -->
         <xsl:when test="contains($context,concat($xpath,'|'))"/>
         <xsl:otherwise><xsl:value-of select="$xpath"/>|</xsl:otherwise>
      </xsl:choose>
   </xsl:function>  
   
   <xsl:function name="cam2func:maskItemsCorrelation">
      <xsl:param name="context"/>
      <xsl:param name="xpath"/>
      <xsl:choose>
         <xsl:when test="contains($context,'*')"/>
         <xsl:when test="ends-with(upper-case($context),'DATES')"/>
         <xsl:when test="ends-with(upper-case($context),'TION')"/>
         <xsl:when test="ends-with(upper-case($context),'DESC')"/>
         <xsl:when test="ends-with(upper-case($context),'ID')"/>
         <xsl:when test="contains($xpath,replace($context,'//','/'))"/>
         <xsl:otherwise><xsl:value-of select="$context"/>|</xsl:otherwise>
      </xsl:choose>
   </xsl:function>
   
   <xsl:function name="cam2func:codeItemsCorrelation">
      <xsl:param name="context"/>
      <xsl:param name="xpath"/>
      <xsl:choose>
         <xsl:when test="contains($context,'*')"/>
         <xsl:when test="ends-with(upper-case($context),'TION')"/>
         <xsl:when test="ends-with(upper-case($context),'DESC')"/>
         <xsl:when test="ends-with(upper-case($context),'ID')"/>
         <xsl:when test="ends-with(upper-case($context),'URN')"/>
         <xsl:when test="contains($xpath,replace($context,'//','/'))"/>
         <xsl:otherwise><xsl:value-of select="$context"/>|</xsl:otherwise>
      </xsl:choose>               
   </xsl:function>
   
   <xsl:function name="cam2func:checkDomain">
      <!-- Select domains to ignore for NDR checking -->
      <xsl:param name="prefix"/>
      <xsl:variable name="domain" select="$rules//as:Namespaces/as:namespace[@prefix=$prefix]/text()"/>
      <xsl:variable name="result">
      <xsl:choose>
         <xsl:when test="$ignoreDomainsNDR='true'"/>
         <xsl:when test="string-length($domain)=0">true</xsl:when>
         <xsl:when test="contains($domain,'urn:oasis:names:')"/>
         <xsl:when test="contains($domain,'www.w3.org')"/>
         <xsl:otherwise>true</xsl:otherwise>
      </xsl:choose>
      </xsl:variable>
      <xsl:value-of select="$result"/>
   </xsl:function>
   
   <xsl:function name="camed:splitNewWords">
      <xsl:param name="words"/>
      <xsl:param name="moreWords"/>
      <xsl:param name="start"/>
      <xsl:if test="string-length($words) &gt; 0">
         <xsl:variable name="firstChar" select="substring($words,1,1)"/>
         <xsl:variable name="nextChars" select="substring-after($words,$firstChar)"/>
         <xsl:variable name="firstNextChar" select="substring($nextChars,1,1)"/>
         <xsl:choose>
            <xsl:when test="contains('.-_',$firstChar)">|<xsl:value-of select="camed:splitNewWords($nextChars,$nextChars,'true')"/></xsl:when>
            <xsl:when test="$start='true'"><xsl:value-of select="upper-case($firstChar)"/><xsl:value-of select="camed:splitNewWords($nextChars,$nextChars,'false')"/></xsl:when>
            <xsl:when test="$firstChar=upper-case(substring($words,1,1)) and($firstNextChar='')"><xsl:value-of select="$firstChar"/></xsl:when>
            <xsl:when test="not(contains('_.-0123456789',$firstNextChar)) and $firstNextChar=lower-case(substring($nextChars,1,1)) and(upper-case($firstChar) = substring($words,1,1))">|<xsl:value-of select="$firstChar"/><xsl:value-of select="camed:splitNewWords($nextChars,$nextChars,'false')"/></xsl:when>
            <xsl:when test="not(contains('_.-0123456789',$firstNextChar)) and($firstNextChar=upper-case(substring($nextChars,1,1)) and(lower-case($firstChar) = substring($words,1,1)))"><xsl:value-of select="$firstChar"/>|<xsl:value-of select="camed:splitNewWords($nextChars,$nextChars,'false')"/></xsl:when>
            <xsl:when test="$firstNextChar=lower-case(substring($nextChars,1,1))"><xsl:value-of select="$firstChar"/><xsl:value-of select="camed:splitNewWords($nextChars,$nextChars,'false')"/></xsl:when>
            <xsl:when test="$start='true'"><xsl:value-of select="upper-case($firstChar)"/><xsl:value-of select="camed:splitNewWords($nextChars,$nextChars,'false')"/></xsl:when>
            <xsl:otherwise><xsl:value-of select="$firstChar"/><xsl:value-of select="camed:splitNewWords($nextChars,$nextChars,'false')"/></xsl:otherwise>
         </xsl:choose>
      </xsl:if>
   </xsl:function>
   
</xsl:stylesheet>
