<?xml version="1.0"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:as="http://www.oasis-open.org/committees/cam" xmlns:saxon="http://saxon.sf.net/" exclude-result-prefixes="saxon">
	<!-- NOTE: this is only BETA code - is not actually used in CAMed - instead Java code writes out the CXF -->
	<xsl:param name="filepath"/>
	<xsl:output method="xml" indent="yes"/>
	<xsl:strip-space elements="*"/>
	<xsl:template match="/">
		<xsl:message select="$filepath"/>
		<as:CAM xmlns:as="http://www.oasis-open.org/committees/cam" CAMlevel="{as:CAM/@CAMlevel}" version="{as:CAM/@version}">
			<xsl:copy-of select=".//namespace::*"/>
			<xsl:apply-templates select="//as:Header"/>
			<xsl:apply-templates select="//as:AssemblyStructure"/>
		</as:CAM>
	</xsl:template>

	<xsl:template match="as:Header">
		<as:Header>
			<as:Description>
				<xsl:value-of select="as:Description"/>
			</as:Description>
			<as:Owner>
				<xsl:value-of select="as:Owner"/>
			</as:Owner>
			<as:Version>
				<xsl:value-of select="as:Version"/>
			</as:Version>
			<as:DateTime>
				<xsl:value-of select="as:DateTime"/>
			</as:DateTime>
			<as:Namespaces>
				<xsl:for-each select="/as:CAM/namespace::*">
					<as:namespace prefix="{./name()}">
						<xsl:value-of select="."/>
					</as:namespace>
				</xsl:for-each>
			</as:Namespaces>
			<xsl:copy-of select="as:Parameters"/>
		</as:Header>
	</xsl:template>

	<xsl:template match="as:AssemblyStructure">
		<xsl:copy>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>



	<xsl:template match="as:Structure">
		<as:Structure ID="{@ID}" reference="{@reference}" taxonomy="{@taxonomy}">
			<as:parameters>
				<xsl:copy-of select="//as:Header/as:Parameters/as:Parameter"/>
			</as:parameters>
			<xsl:variable name="structure">
				<xsl:apply-templates mode="start"/>
			</xsl:variable>
			<xsl:variable name="defaultRules">
				<xsl:apply-templates select="//as:BusinessUseContext/as:Rules/*[contains(name(),'context') or contains(name(),'constraint')]" mode="startRules"/>
			</xsl:variable>
			<xsl:message terminate="yes" select="$defaultRules"/>
			<xsl:apply-templates select="$structure" mode="structure">
				<xsl:with-param name="defaultRules" select="$defaultRules"/>
			</xsl:apply-templates>
		</as:Structure>
	</xsl:template>


	<xsl:template match="*" mode="start">
		<xsl:param name="include"/>
		<xsl:copy>
			<xsl:if test="string-length($include) &gt; 0">
				<xsl:attribute name="as:include" select="$include"/>
			</xsl:if>
			<xsl:apply-templates mode="start"/>
		</xsl:copy>
	</xsl:template>

	<xsl:template match="@*" mode="start">
		<xsl:param name="include"/>
		<xsl:copy-of select="."/>
	</xsl:template>

	<xsl:template match="as:include" mode="start">
		<xsl:variable name="filePath">file://<xsl:value-of select="$filepath"/>/<xsl:value-of select="."/></xsl:variable>
		<xsl:variable name="include" select="document($filePath)"/>
		<xsl:apply-templates select="$include/*" mode="start">
			<xsl:with-param name="include" select="."/>
		</xsl:apply-templates>
	</xsl:template>

	<xsl:template match="as:constraint|as:context|@*" mode="startRules">
		<xsl:variable name="action" select="@action"/>
		<xsl:choose>
			<xsl:when test="contains($action,'applyTemplate')">
				<xsl:variable name="templateName" select="if (contains(substring-after($action,','),',')) then substring-before(substring-after($action,','),',') else substring-before(substring-after($action,','),')')"/>
				<xsl:variable name="templateFileName" select="concat('file://',$filepath,'/',$templateName)"/>
				<xsl:message>templateFileName=<xsl:value-of select="$templateFileName"/></xsl:message>
				<xsl:variable name="template" select="document($templateFileName)"/>
				<xsl:variable name="applyTemplateRules" select="$template//as:BusinessUseContext/as:Rules/as:default/as:context/as:constraint"/>
				<xsl:apply-templates select="$applyTemplateRules" mode="start"/>
				<xsl:copy>
					<xsl:apply-templates mode="startRules"/>
				</xsl:copy>
			</xsl:when>
			<xsl:otherwise>
				<xsl:copy>
					<xsl:apply-templates mode="startRules"/>
				</xsl:copy>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="structure">
		<xsl:param name="path"/>
		<xsl:param name="defaultRules"/>
		<xsl:variable name="namespace-uri" select="namespace-uri()"/>
		<as:Element name="{name()}" path="{concat($path,'/',name())}" namespace="{namespace-uri()}" namespacePrefix="{./namespace::*[string() = $namespace-uri]/name()}">
			<xsl:if test="not(exists(*)) and string-length(text()) &gt; 0">
				<xsl:attribute name="exampleValue" select="text()"/>
			</xsl:if>
			<xsl:if test="@as:include">
				<xsl:attribute name="includedFile" select="@as:include"/>
				<xsl:attribute name="included">true</xsl:attribute>
			</xsl:if>
			<xsl:variable name="rules">
				<xsl:apply-templates select="$defaultRules" mode="default">
					<xsl:with-param name="structure" select="/"/>
					<xsl:with-param name="this" select="name()"/>
					<xsl:with-param name="defaultRules" select="$defaultRules"/>
				</xsl:apply-templates>
			</xsl:variable>
			<as:Rule>
				<xsl:choose>
					<xsl:when test="$rules/as:*">
						<xsl:copy-of select="$rules"/>
					</xsl:when>
					<xsl:otherwise>
						<as:constraint action="makeMandatory()"/>
					</xsl:otherwise>
				</xsl:choose>
			</as:Rule>
			<xsl:apply-templates mode="structure">
				<xsl:with-param name="path" select="concat($path,'/',name())"/>
				<xsl:with-param name="defaultRules" select="$defaultRules"/>
			</xsl:apply-templates>
		</as:Element>
	</xsl:template>


	<xsl:template match="text()" mode="structure"/>

	<xsl:template match="@*" mode="structure">
		<xsl:param name="path"/>
		<xsl:param name="defaultRules"/>
		<as:Attribute name="{name()}" path="{concat($path,'/',name())}">
			<xsl:variable name="rules">
				<xsl:apply-templates select="$defaultRules" mode="default">
					<xsl:with-param name="structure" select="/"/>
					<xsl:with-param name="this" select="name()"/>
					<xsl:with-param name="defaultRules" select="$defaultRules"/>
				</xsl:apply-templates>
			</xsl:variable>
			<as:Rule>
				<xsl:choose>
					<xsl:when test="$rules/as:*">
						<xsl:copy-of select="$rules"/>
					</xsl:when>
					<xsl:otherwise>
						<as:constraint action="makeMandatory()"/>
					</xsl:otherwise>
				</xsl:choose>
			</as:Rule>
		</as:Attribute>
	</xsl:template>

	<xsl:template match="*|@*|text()">
	</xsl:template>

	<xsl:template match="*" mode="rule">
		<xsl:param name="action"/>
		<xsl:element name="as:constraint">
			<xsl:choose>
				<xsl:when test="contains($action,',')">
					<xsl:variable name="newAction" select="concat(substring-before($action,'('),'(',substring-after($action,','))"/>
					<xsl:variable name="xpath" select="substring-before(substring-after($action,'('),',')"/>
					<xsl:attribute name="action" select="$newAction"/>
					<xsl:attribute name="xpath" select="$xpath"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:variable name="newAction" select="concat(substring-before($action,'('),'()')"/>
					<xsl:variable name="xpath" select="substring(substring-after($action,'('),1,string-length(substring-after($action,'('))-1)"/>
					<xsl:attribute name="action" select="$newAction"/>
					<xsl:attribute name="xpath" select="$xpath"/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:element>
	</xsl:template>



	<xsl:template match="as:constraint" mode="default">
		<xsl:param name="structure"/>
		<xsl:param name="this"/>
		<xsl:param name="defaultRules"/>
		<xsl:param name="apply"/>
		<xsl:if test="$apply='true'">
			<xsl:message select="$this"/>
			<xsl:message select="@action"/>
		</xsl:if>
		<!--<xsl:message select="concat($this,'+',@action)"/>-->

		<xsl:variable name="action" select="@action"/>
		<xsl:choose>
			<xsl:when test="contains($action,',') and not(ends-with($action,'])'))">
				<!--				<xsl:message>action="<xsl:value-of select="@action"/>"</xsl:message>-->
				<xsl:variable name="xpathin" select="substring-before(substring-after($action,'('),',')"/>
				<xsl:variable name="xpath" select="concat('$p1',$xpathin)"/>
				<xsl:for-each select="saxon:evaluate($xpath,$structure)">
					<xsl:if test="name()=$this">
						<xsl:apply-templates select="." mode="rule">
							<xsl:with-param name="action" select="$action"/>
						</xsl:apply-templates>
					</xsl:if>
				</xsl:for-each>
			</xsl:when>
			<xsl:otherwise>
				<!--				<xsl:message>action="<xsl:value-of select="@action"/>"</xsl:message>-->
				<xsl:variable name="xpathin" select="substring(substring-after($action,'('),1,string-length(substring-after($action,'('))-1)"/>
				<xsl:variable name="xpath" select="concat('$p1',$xpathin)"/>
				<xsl:for-each select="saxon:evaluate($xpath,$structure)">
					<xsl:if test="name()=$this">
						<xsl:apply-templates select="." mode="rule">
							<xsl:with-param name="action" select="$action"/>
						</xsl:apply-templates>
					</xsl:if>
				</xsl:for-each>
				<xsl:fallback>
				</xsl:fallback>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
</xsl:stylesheet><!-- Stylus Studio meta-information - (c) 2004-2007. Progress Software Corporation. All rights reserved.

<metaInformation>
	<scenarios>
		<scenario default="yes" name="Scenario1" userelativepaths="yes" externalpreview="no" url="..\..\..\..\ManageWidget\cam\generatedWidget.cam" htmlbaseurl="" outputurl="" processortype="saxon8" useresolver="yes" profilemode="0" profiledepth=""
		          profilelength="" urlprofilexml="" commandline="" additionalpath="" additionalclasspath="" postprocessortype="none" postprocesscommandline="" postprocessadditionalpath="" postprocessgeneratedext="" validateoutput="no" validator="internal"
		          customvalidator=""/>
	</scenarios>
	<MapperMetaTag>
		<MapperInfo srcSchemaPathIsRelative="yes" srcSchemaInterpretAsXML="no" destSchemaPath="" destSchemaRoot="" destSchemaPathIsRelative="yes" destSchemaInterpretAsXML="no"/>
		<MapperBlockPosition></MapperBlockPosition>
		<TemplateContext></TemplateContext>
		<MapperFilter side="source"></MapperFilter>
	</MapperMetaTag>
</metaInformation>
-->